import os
from urllib import parse

from flask import Flask
from flask_apscheduler import APScheduler
from flask_cors import CORS

from sqlalchemy import create_engine
from app.api import api
from app.helpers import request_handlers
from config import Config

manage_path = os.path.dirname(os.path.abspath(__file__))
config_yml_path = os.path.join(os.path.dirname(os.path.dirname(manage_path)), 'config', 'config.yml')


class ApschedulerConfig(object):
    # 开关
    SCHEDULER_API_ENABLED = True

    SCHEDULER_EXECUTORS = {
        'default': {'type': 'threadpool', 'max_workers': 20}
    }


def create_app():
    """创建app并初始化相关配置参数"""

    config = Config()
    config.load(config_yml_path)

    app = Flask(__name__)

    app.config.from_object(ApschedulerConfig())

    CORS(app)

    request_handlers.configure(app)

    url = f'postgresql+psycopg2://{config.database.USER}:{parse.quote(config.database.PASSWORD)}@{config.database.HOST}:{config.database.PORT}/{config.database.DB}'
    app.engine = create_engine(url)

    api.init_app(app)

    app.scheduler = APScheduler()
    app.scheduler.init_app(app)
    app.scheduler.start()

    return app
